function Q = buildQ(S,N,Call,Put);

% Build the Arrow Debreu state prices for the implied trinomial tree.
% Inputs:
%   N    = Number of time steps (scalar).
%   S    = Vector of strike prices.  Vector of dimension 2N+1
%   Call = Market call prices.  Matrix 2N+1 by N+1.
%   Put  = Market put prices.  Matrix 2N+1 by N+1.

% Initialize the Arrow-Debreu state prices, Q.
Q = zeros(2*N+1,N+1);

% Middle node, time zero.
Q(N+1,1) = Call(N+2,1)/(S(N+1) - S(N+2));

% Construct the tree for the Arrow-Debreu prices, Q.
% Upper Nodes of the tree for Q(i,j).
for j=2:N+1
	for i=N-j+2:N+1;
		SumUp = 0;
		for k=1:i-1
			SumUp = SumUp + Q(k,j)*(S(k)-S(i+1));
		end
		Q(i,j) = (Call(i+1,j) - SumUp) / (S(i) - S(i+1));
	end
end

% Lower Nodes of the tree for Q(i,j).
for j=2:N+1
	for i=N+j:-1:N+2
		SumLow = 0;
		for k=N+j:-1:i+1
			SumLow = SumLow + Q(k,j)*(S(i-1) - S(k));
		end
		Q(i,j) = (Put(i-1,j) - SumLow) / (S(i-1) - S(i));
	end
end
